# Qgis 2.99 master (as of 2017-03-20)
[<-- Main 'Table of content'](Home#table_of_content)
 
[<-- 'Building of QGIS 3.* versions'](Building-QGIS-3#topics)

[<-- 'Building QGIS 3.0 from scratch'](Building-QGIS-3.0-from-scratch#topics)

[<-- 'Building QT 5 from scratch'](Building-QT-5-from-scratch#topics)

***

[--> 'Download the source'](#download_source)

[--> 'Configuration'](#configuration)

[--> 'Build and install'](#build)

[--> 'Sample Script to set environment for ccmake'](#ccmake_paths)

---

**Download the source**
<a name="download_source"/>

**Step 1**) create the directory where the QGIS 3.0 source should be stored:

```c
mkdir git_qgis
cd git_qgis
```

**Step 2**) Clone the git repository:
* this can take a while to compleate
```c
git clone git@github.com:qgis/QGIS.git
```

You should now have a **QGIS** directory
```c
cd QGIS
```

**Step 3**) checkout the QGIs branch needed (here master)
* with the QT git, we used `git tag`, here we must use `git branch`

**parameters**
* `-a` will show the local and remote branches
* `-r` will show only the remote branches (without `-a` or `-r` only the local branches)
* `-vv` will also show the last commit text

If you wish to see only the 'release' branches_
```c
git branch -a -vv | grep release

  remotes/origin/release-0_0_11    20bbfac This commit was manufactured by cvs2svn to create tag 'Release-0_0_11'.
  remotes/origin/release-0_0_12    137561e This commit was manufactured by cvs2svn to create tag 'Release-0_0_12'.
...
  remotes/origin/release-2_16      fd081c8 [style dock] emit widgetChanged() when raster palette labels updated
  remotes/origin/release-2_18      af74635 fix build (followup 737719e)
  remotes/origin/release-2_4       fe0a303 Merge pull request #1526 from jarl-dk/release-2_4
  remotes/origin/release-2_6       22efb06 backport e04570cc89d30f38430a5e71cfb795971c40afe0
  remotes/origin/release-2_8       44c948b [processing] add support for GDAL 2.x in raster tools (refs #14858)
```

In most cases the `master` is needed
* and is normally downloaded and activated (`checkout`) during the `clone`

```c
git branch -vv

* master 4b041d6 [origin/master] Improve docs
  origin 4b041d6 Improve docs

```

The `*` before the branch-name implies which branch has been checked-out (i.e. is active)

**Step 3**) create a build directory
* this directory will contain all the files needed to build and install the project
   * without touching the original source code

```c
mkdir build_master
cd build_master
```

**Tips**: before calling **ccmake**
* to insure that ccmake can find everything you need, your environment must be properly set
   * this is especially true when you have irregular install paths
   * setting **PATH**, **LD_LIBRARY_PATH** and **PYTHONHOME** properly 
      * and in **the correct order** is important 
         * [Sample Script to set environment for ccmake](#ccmake_paths)
      * when I did this the first time, ccmake asked 13 times where each of the QT5 libraies were
         * this time all of the QT5 libraies were found
* A simple way to see if **ccmake** will find your **intended** QT5 installation
  * is to call **qmake -v**

```c
qmake -v
QMake version 3.0
Using Qt version 5.7.1 in /home/mj10777/000_links/apps_local/libQt571/lib
```

***

**Configuration**
<a name="configuration"/>

From **inside** the build directory
With
**ccmake ..**  will be called

* `..` tells ccmake to read the `QGIS/CMakeLists.txt` file that is needed

```c
ccmake ..
```

A console dialog will opened and show **EMPTY CACHE**

Further down
* **Press [c] to configure**
   * `c` will start the configuration looking for everything it needs
      * if something cannot be found: `Errors occurred during the last pass`will be shown
         * with a corresponding error text
   * `Press [e] to exit help` 
      * will show the configuration dialog, where corrections can be made

```c
 CMake Error at cmake/FindQCA.cmake:59 (message):
   Could not find QCA
 Call Stack (most recent call first):
   CMakeLists.txt:282 (FIND_PACKAGE)
```

After pressing `e`
* a list of the parameters (left) and the values (right) will be shown
   * look for (in this case) for **QCA**

```c
 QCA_INCLUDE_DIR                 /usr/include/QtCrypto                                                                               
 QCA_LIBRARY                     *QCA_LIBRARY-NOTFOUND 
```

Here, ccmake, found the QT4 version of this library
* caused by the source installing the `include` files in **libQt571/include/Qca-qt5/QtCrypto**

Adapt the entries to the correct values:

```c
 QCA_INCLUDE_DIR                 /home/mj10777/000_links/apps_local/libQt571/include/Qca-qt5/QtCrypto                                
 QCA_LIBRARY                     /home/mj10777/000_links/apps_local/libQt571/lib/libqca-qt5.so
```

Here I noticed that for `QScintilla` 
* the correct `include` directory was found
   * but the QT 4 version of the library

```c
 QSCINTILLA_INCLUDE_DIR          /home/mj10777/000_links/apps_local/libQt571/include                                                 
 QSCINTILLA_LIBRARY              /usr/lib/x86_64-linux-gnu/libqscintilla2.so 
```

**Warning:** never mix the QT 4 and QT 5
* everything should be **QT 5**

```c
 QSCINTILLA_INCLUDE_DIR          /home/mj10777/000_links/apps_local/libQt571/include                                                 
 QSCINTILLA_LIBRARY              /home/mj10777/000_links/apps_local/libQt571/lib/libqscintilla2_qt5.so
```

Now press `c` again to continue the configuration
* now it complains that it cannot find **QWT_INCLUDE_DIR**, which was also install

```c
 QWT_INCLUDE_DIR                  /home/mj10777/000_links/apps_local/libQt571/qwt-6.1.3/include                                       
 QWT_LIBRARY                      /home/mj10777/000_links/apps_local/libQt571/lib/libqwt.so
```

Now press `c` again to continue the configuration
* now completes correctly and shows a new menue: **Press [g] to generate and exit**

Before pressing `g`, check all the entries
* that everything was correctly found
   * i.e. no QT4 libraries etc.
* `Press [t] to toggle advanced mode` will show you the complete list

```c
 WITH_ASTYLE                      OFF
```
Should be set to **ON**

After any change, `c` must again be pressed
***

Press `g` again to **complete** the configuration
* all of the needed make files, scripts will be created inside the build-directory

Here I received 8 warnings that looked like this:
```c
 WARNING: Target "qgis_core" requests linking to directory "/home/mj10777/000_links/apps_local/libQt571/lib".  Targets may link only
 to libraries.  CMake is dropping the item.
```

**Cause**: for **QWT_LIBRARY** and **QCA_LIBRARY*
*  /home/mj10777/000_links/apps_local/libQt571/lib

instead of:
* /home/mj10777/000_links/apps_local/libQt571/lib/**libqwt.so**
* /home/mj10777/000_links/apps_local/libQt571/lib/**libqca-qt5.so**

At some point, no more **errors** or **warnings** will occur
* and ccmake will close


***

**Build and install**
<a name="build"/>

```c
make
```

At some point make will complete
* with no errors

You can the install the results with:
* **Note:** in this scenario **sudo** is not needed

```c
make install
```

**Sample Script to set environment for ccmake**
<a name="ccmake_paths"/>

Open in terminal in the build directory
* call with **source ccmake_paths.sh**

```bash
#/bin/bash
#--------------------------------------
# --> set environment and work in terminal
# source ccmake_paths.sh
#--------------------------------------
# path where local applications are stored
LINKS_PATH="/home/mj10777/000_links/apps_local";
# System libs (will be set at the end)
# and using other default libs (spatialite, geos, proj4 etc.)
# libraries of localy compiled projects
SYSTEM_LIB_PATH="/usr/local/lib:/usr/lib";
#--------------------------------------
# setting QGIS
QGIS_PATH="${LINKS_PATH}/qgis_300";
# setting QT
QT_PATH="${LINKS_PATH}/libQt571";
# setting GDAL
GDAL_PATH="${LINKS_PATH}/gdal_git_master";
# setting external libs used by gdal [LIBKML, MrSID, FileGDB, libecwj2]
GDAL_EXTERNALS_PATH="${LINKS_PATH}/gdal_externals";
#---------------------------------------------------------
# -- no more changes in the script should be needed --
#---------------------------------------------------------
START_DIR=${PWD};
SCRIPT_PATH="`dirname \"$0\"`";
BASE_NAME_SCRIPT=`basename $0`;
BASE_NAME_SCRIPT_CUT=`basename $0 | cut -d '.' -f1`;
SET_ENVIRONMENT="";
case ${BASE_NAME_SCRIPT_CUT} in
 bash | source)
 # [bash] this has been called as 'source script_name.sh'
 # [source] 'source script_name.sh' has been called from a script-name that starts with 'source.' : source.qgis_300_gdal_git_master.sh
 SET_ENVIRONMENT="set_environment";
 SCRIPT_PATH=${START_DIR};
 ;;
esac
#--------------------------------------
PYTHON27_VERSION="python2.7";
PYTHON34_VERSION="python3.4";
PYTHON_VERSION="${PYTHON34_VERSION}";
#--------------------------------------
# using GDAL-git-master binaries
GDAL_BIN_PATH="${GDAL_PATH}/bin";
GDAL_LIB_PATH="${GDAL_PATH}/lib";
#--------------------------------------
# using QT  binaries
QT_BIN_PATH="${QT_PATH}/bin";
QT_LIB_PATH="${QT_PATH}/lib";
#--------------------------------------
PYTHON34_QT5_DIR="${LINKS_PATH}/libQt571";
PYTHON34_QT5_BASE="${PYTHON34_QT5_DIR}/lib/${PYTHON34_VERSION}";
PYTHON34_QT5_PACKAGES="${PYTHON34_QT5_BASE}:${PYTHON34_QT5_BASE}/site-packages:${PYTHON34_QT5_BASE}/dist-packages:${PYTHON34_QT5_BASE}/lib-dynload:${PYTHON34_QT5_BASE}/plat-x86_64-linux-gnu";
#--------------------------------------
PYTHON_HOME="${PYTHON34_QT5_DIR}";
#---------------------------------------------------------
# using QGIS  binaries
QGIS_BIN_PATH="${QGIS_PATH}/bin";
QGIS_LIB_PATH="${QGIS_PATH}/lib";
QGIS_SHARE_PATH="${QGIS_PATH}/share/qgis";
QGIS_PYTHON_PATH="${QGIS_SHARE_PATH}/python";
# using GDAL-git-master binaries
GDAL_BIN_PATH="${GDAL_PATH}/bin";
GDAL_LIB_PATH="${GDAL_PATH}/lib";
# using QT  binaries
QT_BIN_PATH="${QT_PATH}/bin";
QT_LIB_PATH="${QT_PATH}/lib";
GDAL_PYTHON_PATH="${GDAL_LIB_PATH}/python3.4/site-packages";
if [ ! -z "${GDAL_EXTERNALS_PATH}" ]
then
 # adding external libs used by gdal, so that gdal can find it
 GDAL_LIB_PATH="${GDAL_LIB_PATH}:${GDAL_EXTERNALS_PATH}/lib";
fi
GDAL_SHARE_PATH="${GDAL_PATH}/share/gdal";
QT_PYTHON_PATH="${PYTHON34_QT5_PACKAGES}";
#---------------------------------------------------------
# using QGIS binaries
BIN_PATH="${QGIS_BIN_PATH}";
# using gdal binaries
BIN_PATH="${BIN_PATH}:${GDAL_BIN_PATH}";
# /python
# using QGIS python needed by QGIS  binaries
PYTHON_PATH="${QGIS_PYTHON_PATH}";
PYTHON_PATH="${PYTHON_PATH}:${QT_PYTHON_PATH}";
PYTHON_PATH="${PYTHON_PATH}:${GDAL_PYTHON_PATH}";
# /share
# using QGIS share needed by QGIS  binaries
SHARE_PATH="${QGIS_SHARE_PATH}";
# using gdal share needed by QGIS  binaries
SHARE_PATH="${SHARE_PATH}:${GDAL_SHARE_PATH}";
# /lib
# using QGIS libs needed by QGIS  binaries
LIB_PATH="${QGIS_LIB_PATH}";
# using gdal libs needed by QGIS binaries
LIB_PATH="${LIB_PATH}:${GDAL_LIB_PATH}";
# using QT libs needed by QGIS binaries
LIB_PATH="${LIB_PATH}:${QT_LIB_PATH}";
# default system libraries
LIB_PATH="${LIB_PATH}:${SYSTEM_LIB_PATH}";
# set export, so that the binarie will use this
# using QGIS 2.16 libs needed by QGIS 2.16 binaries
# PYTHON_PATH="${PYTHON_PATH}:${PYTHONPATH}";
BIN_PATH="${BIN_PATH}:${PATH}";
# --------------------------------------
export LD_LIBRARY_PATH="${LIB_PATH}";
export PYTHONPATH="${PYTHON_PATH}";
export PYTHONHOME="${PYTHON_HOME}"
export PATH="${BIN_PATH}";
export GDAL_DATA="${GDAL_SHARE_PATH}";
export QGIS_PREFIX_PATH="${QGIS_PATH}";
#------------------------------------------------------------------------------
RED='\033[0;31m';
BRED='\033[1;31m'         # Red
BOLD='\033[1m'  # \x1B[31m
CYAN='\033[0;36m';
BCYAN='\033[1;36m'        # Cyan
BROWN='\033[0;33m';
BBROWN='\033[1;33m'        # Cyan
GREEN='\033[0;32m';
BGREEN='\033[1;32m'       # Green
NC='\033[0m' # No Color
# echo -e "\033[1mThis is bold text.\033[0m"
#------------------------------------------------------------------------------
BENE="${GREEN}'Tutto bene!'${NC}";
BENENO="${BRED}'No bene!'${NC}";
HABE_FERTIG="${BOLD}Ich habe fertig.${NC}";
MTI="${GREEN}-I->${NC}";
MTIII="${CYAN}>I>>${NC}";
MTII="${RED}-I>>${NC}";
MTW="${BROWN}-W->${NC}";
MTWW="${BROWN}-W>>${NC}";
MTWWW="${BROWN}>W>>${NC}";
MTE="${BRED}-E->${NC}";
MTEE="${RED}-E>>${NC}";
MTEEE="${RED}>E>>${NC}";
MTH="${CYAN}-H->${NC}";
ARROW="${CYAN}---->${NC}";
MTD="${BROWN}>>D->${NC}";
MESSAGE_TYPE=${MTI};
exit_rc=0;
#--------------------------------------
echo -e "${BOLD}-------> ${BASE_NAME_SCRIPT_CUT}:\n${MTII} ${BOLD}share_path[${NC}${CYAN}${SHARE_PATH}${NC}${BOLD}]${NC}";
#--------------------------------------
# grep -i [case insensitive]
GREP_SEARCH="'${LINKS_PATH}\|libqt\|libgdal\|libkml\|libqgis\|libspatialite\|libproj\|libgeos\|libpng\|libjpeg\|libcairo\|libpixman'";
echo -e "${MTII} ${BOLD}------->${NC}\n";
echo -e "${CYAN}export ${NC}${BOLD}PATH${NC}=\"${BIN_PATH}\"${NC}";
echo -e "${GREEN}export ${NC}${BOLD}LD_LIBRARY_PATH${NC}=\"${LIB_PATH}\"${NC}";
echo -e "${CYAN}export ${NC}${BOLD}PYTHONPATH${NC}=\"${PYTHON_PATH}\"${NC}";
echo -e "${GREEN}export ${NC}${BOLD}PYTHONHOME${NC}=\"${PYTHONHOME}\"${NC}";
echo -e "${CYAN}export ${NC}${BOLD}GDAL_DATA${NC}=\"${GDAL_SHARE_PATH}\"${NC}";
echo -e "${GREEN}export ${NC}${BOLD}QGIS_PREFIX_PATH${NC}=\"${QGIS_PREFIX_PATH}\"${NC}";
echo -e "${BRED}#  ${QGIS_BIN}${NC}\n${BOLD}ldd -v \$(which qgis) | grep -i ${GREP_SEARCH}${NC}\n";
echo -e "${BOLD}<-------${NC}\n";
#--------------------------------------
if [ -z "${SET_ENVIRONMENT}" ]
then
 #--------------------------------------
  MESSAGE_TEXT="${MTI} Qgis has been started with the set ${BOLD}environment${NC}\n ${MTII} the default environment settings are again active \n";
  echo -e "${MESSAGE_TEXT}";
 #--------------------------------------
 exit ${exit_rc};
 #--------------------------------------
else
 #---------------------------------------------------------
 # run in a terminal with set export values
 # called as 'source script_name.sh'
 # - NEVER use 'exit' when called with 'source'
 # -> terminal will close ('exit')
 #---------------------------------------------------------
 case ${BASE_NAME_SCRIPT_CUT} in
  bash)
   MESSAGE_TEXT="${MTI} The terminal ${BOLD}environment${NC} in [${GREEN}${SCRIPT_PATH}${NC}]\n ${MTII} has been set to use the above settings \n";
  ;;
  *)
   BASE_NAME_SCRIPT_SUFFIX=${BASE_NAME_SCRIPT#${BASE_NAME_SCRIPT_CUT}.}
   MESSAGE_TEXT="${MTI} The script ${BOLD}environment${NC} in [${GREEN}${SCRIPT_PATH}${NC}]\n ${MTII} has been set to use the above settings\n  ${MTWW} not set for the terminal-environment that called [${BASE_NAME_SCRIPT}]\n   ${MTIII} to set the terminal-environment directly, you must use '${BOLD}source ${NC} ${GREEN}${BASE_NAME_SCRIPT_SUFFIX}${NC}'";
  ;;
 esac
 echo -e "${MESSAGE_TEXT}";
fi
```

---
2017-03-20: Mark Johnson, Berlin Germany

---