# Starting QGIS3 as seen from lddtree.sh
[<-- Main 'Table of content'](Home#table_of_content)
 
[<-- 'Building of QGIS 3.* versions'](Building-QGIS-3#topics)

[<-- 'Building QGIS 3.0 from scratch'](Building-QGIS-3.0-from-scratch#topics)

[<-- 'Building QT 5 from scratch'](Building-QT-5-from-scratch#topics)

[<-- 'Building QGIS3 with ccmake'](QGIS3-ccmake)


***

[--> 'Download the lddtree.sh being used here'](#lddtree_source)

[--> 'Starting QGIS3 for the first time'](#qgis_start_first)

[--> 'Reported problem PyQt5 fails'](#pyqt5_fails)

[--> 'Calling lddtree.sh with QGIS3'](#lddtree_qgis)

---
**Note:** 
* although this page is specifically for QGIS3 and QT5
   * many of the methods shown here apply to other applications with complicated dependencies 

---

**Download the lddtree.sh being used here**
<a name="lddtree_source"/>

* [Fork of pax-util lddtree.sh](https://github.com/ncopa/lddtree)

```bash
wget https://raw.githubusercontent.com/ncopa/lddtree/master/lddtree.sh
```

Normally **ldd* is used to print shared object dependencies
* combined with the '-v' the output is printed in a tree form 

There is also a pax-utils **lddtree** which can be used.

The above fork brings (for me) a better result
* and when sent to file, retains the tree structure better

***

**Starting QGIS3 for the first time**
<a name="qgis_start_first"/>

As is often the case, starting a complex application
* often fails for unknown reasons

After [compiling and installing QGIS3](QGIS3-ccmake)
* I first started QGIS3 in a terminal
   * which ended without any warnings or errors (i.e. did not start)

This is often caused by some library mixup
* incorrect environment setting
* or sub-library calling incorrect library's

So the next step was to call QGIS3 with `valgrind` 
* `valgrind` reported a crash with QT4 library **libQtCore.so.4**
   * which is definitely a **no, no**

So the task at hand is to find out
* who called that library

```bash
lddtree.sh $(which qgis)
```

Here a relevant portion of the result:

```c
qgis => /home/mj10777/000_links/apps_local/qgis_300/bin/qgis (interpreter => /lib64/ld-linux-x86-64.so.2)
  libqgis_app.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_app.so.2.99.0
    libqwt.so.6 => /home/mj10777/000_links/apps_local/libQt571/lib/libqwt.so.6
      libQt5OpenGL.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5OpenGL.so.5
...
  libqgis_gui.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_gui.so.2.99.0
    libqca.so.2 => /usr/lib/x86_64-linux-gnu/libqca.so.2
      libQtCore.so.4 => /usr/lib/x86_64-linux-gnu/libQtCore.so.4
        libglib-2.0.so.0 => /lib/x86_64-linux-gnu/libglib-2.0.so.0
  libqgis_core.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_core.so.2.99.0
    libspatialindex.so.3 => /usr/local/lib/libspatialindex.so.3
...
```

* **libqwt.so.6** is needed by QGIS3 and was found to use a **libQt5** library
* **libqca.so.2** is needed by QGIS3 and was found to use a **libQt4** library
   * this lead to the [Building QCA for QT 5 - Version 2.1.3](Building-QCA-for-QT-5)
      * after-which QGIS3 started correctly
          * until **python3** found something it did not like

***

**Reported problem PyQt5 fails**
<a name="pyqt5_fails"/>

```c
On Ubuntu 17.04, which is based on Qt5.7.1, output from
`./output/bin/qgis`

`Couldn't load PyQt.
Python support will be disabled. (a really big deal)

Traceback (most recent call last):
   File "", line 1, in
ImportError: /usr/lib/x86_64-linux-gnu/libQt5Core.so.5: version `Qt_5.8'
not found (required by
/usr/local/lib/python3.5/dist-packages/PyQt5/QtCore.so)`
```

**Note:**
* I attempted to use **Python-3.5.3rc1**, failing with some sort of `async` error
   * having no idea what the cause was, I installed **Python-3.4.6** instead
       * to the same directory where QT 5.7.1 was installed

So for me the **QtCore.so** is found in:
```c
/home/mj10777/000_links/apps_local/libQt571/lib/python3.4/site-packages/PyQt5/QtCore.so
```

```bash
lddtree.sh /home/mj10777/000_links/apps_local/libQt571/lib/python3.4/site-packages/PyQt5/QtCore.so
```

The returned result is:
```bash
QtCore.so => /home/mj10777/000_links/apps_local/libQt571/lib/python3.4/site-packages/PyQt5/QtCore.so (interpreter => none)
  libQt5Core.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Core.so.5
    libpthread.so.0 => /lib/x86_64-linux-gnu/libpthread.so.0
    libz.so.1 => /usr/local/lib/libz.so.1
    libicui18n.so.52 => /usr/lib/x86_64-linux-gnu/libicui18n.so.52
    libicuuc.so.52 => /usr/lib/x86_64-linux-gnu/libicuuc.so.52
      libicudata.so.52 => /usr/lib/x86_64-linux-gnu/libicudata.so.52
    libdl.so.2 => /lib/x86_64-linux-gnu/libdl.so.2
    libglib-2.0.so.0 => /lib/x86_64-linux-gnu/libglib-2.0.so.0
      libpcre.so.3 => /lib/x86_64-linux-gnu/libpcre.so.3
    librt.so.1 => /lib/x86_64-linux-gnu/librt.so.1
    libm.so.6 => /lib/x86_64-linux-gnu/libm.so.6
    libgcc_s.so.1 => /lib/x86_64-linux-gnu/libgcc_s.so.1
    ld-linux-x86-64.so.2 => /lib/x86_64-linux-gnu/ld-linux-x86-64.so.2
  libstdc++.so.6 => /usr/lib/x86_64-linux-gnu/libstdc++.so.6
  libc.so.6 => /lib/x86_64-linux-gnu/libc.so.6
```

This shows that **QtCore.so** uses the same **libQt5Core.so.5*, in the same directory
* as QGIS3 does when started (see below)

Here I would suggest that you check if this is the same for you
* using the same QT 5 files (in the same directory) as QGIS3 does

**Note:** I am not a python expert, so my suggestions in python matters may be faulty.

The **PyQt5/QtCore.so** is a library that defines an function-interface between
* **Python-3.4.6** and QtCore 5.7.1

If the **PyQt5/QtCore.so** was created from a different version of QT5 that is being used when called
* that **could** be a cause of your problem

Calling **lddtree.sh** against both
* QGIS3
* PyQt5/QtCore.so (with full path)

and checking if they both use the same libQt5Core.so.5 (or not)
* could help you resolve this problem

***

**Calling lddtree.sh with QGIS3**
<a name="lddtree_qgis"/>
* where QGIS3 starts correctly whithout warnings or errors

Open in a terminal in which one would normally start QGIS3
* setting environment variables as needed
   * [**PATH**, **LD_LIBRARY_PATH**, **PYTHONPATH** and **PYTHONHOME**](QGIS3-ccmake#ccmake_paths)

**Note:** The original indentation of 4 spaces has been reduced to 2 for this page. 

```bash
lddtree.sh $(which qgis)
```

```c
qgis => /home/mj10777/000_links/apps_local/qgis_300/bin/qgis (interpreter => /lib64/ld-linux-x86-64.so.2)
  libqgis_app.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_app.so.2.99.0
    libqwt.so.6 => /home/mj10777/000_links/apps_local/libQt571/lib/libqwt.so.6
      libQt5OpenGL.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5OpenGL.so.5
    libQt5Sql.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Sql.so.5
    libQt5WebKit.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5WebKit.so.5
      libpthread.so.0 => /lib/x86_64-linux-gnu/libpthread.so.0
      librt.so.1 => /lib/x86_64-linux-gnu/librt.so.1
      libz.so.1 => /usr/local/lib/libz.so.1
      libXrender.so.1 => /usr/lib/x86_64-linux-gnu/libXrender.so.1
      libXcomposite.so.1 => /usr/lib/x86_64-linux-gnu/libXcomposite.so.1
      libX11.so.6 => /usr/lib/x86_64-linux-gnu/libX11.so.6
        libxcb.so.1 => /usr/lib/x86_64-linux-gnu/libxcb.so.1
          libXau.so.6 => /usr/lib/x86_64-linux-gnu/libXau.so.6
          libXdmcp.so.6 => /usr/lib/x86_64-linux-gnu/libXdmcp.so.6
        libdl.so.2 => /lib/x86_64-linux-gnu/libdl.so.2
      libxslt.so.1 => /usr/lib/x86_64-linux-gnu/libxslt.so.1
      libxml2.so.2 => /usr/lib/x86_64-linux-gnu/libxml2.so.2
        liblzma.so.5 => /lib/x86_64-linux-gnu/liblzma.so.5
      libicui18n.so.52 => /usr/lib/x86_64-linux-gnu/libicui18n.so.52
      libicuuc.so.52 => /usr/lib/x86_64-linux-gnu/libicuuc.so.52
        libicudata.so.52 => /usr/lib/x86_64-linux-gnu/libicudata.so.52
      libQt5Sensors.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5Sensors.so.5
      libQt5Positioning.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5Positioning.so.5
      libQt5Quick.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5Quick.so.5
      libQt5Qml.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5Qml.so.5
      libQt5WebChannel.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5WebChannel.so.5
      libQt5Multimedia.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5Multimedia.so.5
        libpulse.so.0 => /usr/lib/x86_64-linux-gnu/libpulse.so.0
          libjson-c.so.2 => /lib/x86_64-linux-gnu/libjson-c.so.2
          libpulsecommon-4.0.so => /usr/lib/x86_64-linux-gnu/pulseaudio/libpulsecommon-4.0.so
            libwrap.so.0 => /lib/x86_64-linux-gnu/libwrap.so.0
              libnsl.so.1 => /lib/x86_64-linux-gnu/libnsl.so.1
            libsndfile.so.1 => /usr/lib/x86_64-linux-gnu/libsndfile.so.1
              libFLAC.so.8 => /usr/lib/x86_64-linux-gnu/libFLAC.so.8
              libvorbisenc.so.2 => /usr/lib/x86_64-linux-gnu/libvorbisenc.so.2
              libvorbis.so.0 => /usr/lib/x86_64-linux-gnu/libvorbis.so.0
              libogg.so.0 => /usr/lib/x86_64-linux-gnu/libogg.so.0
            libasyncns.so.0 => /usr/lib/x86_64-linux-gnu/libasyncns.so.0
              libresolv.so.2 => /lib/x86_64-linux-gnu/libresolv.so.2
          libdbus-1.so.3 => /lib/x86_64-linux-gnu/libdbus-1.so.3
      libGL.so.1 => /usr/lib/x86_64-linux-gnu/mesa/libGL.so.1
        libexpat.so.1 => /lib/x86_64-linux-gnu/libexpat.so.1
        libglapi.so.0 => /usr/lib/x86_64-linux-gnu/libglapi.so.0
        libXext.so.6 => /usr/lib/x86_64-linux-gnu/libXext.so.6
        libXdamage.so.1 => /usr/lib/x86_64-linux-gnu/libXdamage.so.1
        libXfixes.so.3 => /usr/lib/x86_64-linux-gnu/libXfixes.so.3
        libX11-xcb.so.1 => /usr/lib/x86_64-linux-gnu/libX11-xcb.so.1
        libxcb-glx.so.0 => /usr/lib/x86_64-linux-gnu/libxcb-glx.so.0
        libxcb-dri2.so.0 => /usr/lib/x86_64-linux-gnu/libxcb-dri2.so.0
        libxcb-dri3.so.0 => /usr/lib/x86_64-linux-gnu/libxcb-dri3.so.0
        libxcb-present.so.0 => /usr/lib/x86_64-linux-gnu/libxcb-present.so.0
        libxcb-sync.so.1 => /usr/lib/x86_64-linux-gnu/libxcb-sync.so.1
        libxshmfence.so.1 => /usr/lib/x86_64-linux-gnu/libxshmfence.so.1
        libXxf86vm.so.1 => /usr/lib/x86_64-linux-gnu/libXxf86vm.so.1
        libdrm.so.2 => /usr/lib/x86_64-linux-gnu/libdrm.so.2
    libQt5WebKitWidgets.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5WebKitWidgets.so.5
      libQt5MultimediaWidgets.so.5 => /media/mj10777/tb_4/apps_local/libQt571/lib/libQt5MultimediaWidgets.so.5
    libgdal.so.20 => /home/mj10777/000_links/apps_local/gdal_2000.master/lib/libgdal.so.20
      libodbc.so.2 => /usr/local/lib/libodbc.so.2
        libltdl.so.7 => /usr/lib/x86_64-linux-gnu/libltdl.so.7
      libodbcinst.so.2 => /usr/local/lib/libodbcinst.so.2
      libxerces-c.so.28 => /usr/lib/libxerces-c.so.28
      libopenjp2.so.6 => /usr/local/lib/libopenjp2.so.6
      libjpeg.so.8 => /usr/local/lib/libjpeg.so.8
      libpq.so.5 => /usr/lib/x86_64-linux-gnu/libpq.so.5
        libssl.so.1.0.0 => /lib/x86_64-linux-gnu/libssl.so.1.0.0
        libcrypto.so.1.0.0 => /lib/x86_64-linux-gnu/libcrypto.so.1.0.0
        libgssapi_krb5.so.2 => /usr/lib/x86_64-linux-gnu/libgssapi_krb5.so.2
          libkrb5.so.3 => /usr/lib/x86_64-linux-gnu/libkrb5.so.3
            libkeyutils.so.1 => /lib/x86_64-linux-gnu/libkeyutils.so.1
          libk5crypto.so.3 => /usr/lib/x86_64-linux-gnu/libk5crypto.so.3
          libcom_err.so.2 => /lib/x86_64-linux-gnu/libcom_err.so.2
          libkrb5support.so.0 => /usr/lib/x86_64-linux-gnu/libkrb5support.so.0
        libldap_r-2.4.so.2 => /usr/lib/x86_64-linux-gnu/libldap_r-2.4.so.2
          liblber-2.4.so.2 => /usr/lib/x86_64-linux-gnu/liblber-2.4.so.2
          libsasl2.so.2 => /usr/lib/x86_64-linux-gnu/libsasl2.so.2
          libgssapi.so.3 => /usr/lib/x86_64-linux-gnu/libgssapi.so.3
            libheimntlm.so.0 => /usr/lib/x86_64-linux-gnu/libheimntlm.so.0
            libkrb5.so.26 => /usr/lib/x86_64-linux-gnu/libkrb5.so.26
              libwind.so.0 => /usr/lib/x86_64-linux-gnu/libwind.so.0
              libheimbase.so.1 => /usr/lib/x86_64-linux-gnu/libheimbase.so.1
              libhx509.so.5 => /usr/lib/x86_64-linux-gnu/libhx509.so.5
              libcrypt.so.1 => /lib/x86_64-linux-gnu/libcrypt.so.1
            libasn1.so.8 => /usr/lib/x86_64-linux-gnu/libasn1.so.8
            libhcrypto.so.4 => /usr/lib/x86_64-linux-gnu/libhcrypto.so.4
            libroken.so.18 => /usr/lib/x86_64-linux-gnu/libroken.so.18
          libgnutls-deb0.so.28 => /usr/lib/x86_64-linux-gnu/libgnutls-deb0.so.28
            libp11-kit.so.0 => /usr/lib/x86_64-linux-gnu/libp11-kit.so.0
              libffi.so.6 => /usr/lib/x86_64-linux-gnu/libffi.so.6
            libtasn1.so.6 => /usr/lib/x86_64-linux-gnu/libtasn1.so.6
            libnettle.so.4 => /usr/lib/x86_64-linux-gnu/libnettle.so.4
            libhogweed.so.2 => /usr/lib/x86_64-linux-gnu/libhogweed.so.2
            libgmp.so.10 => /usr/lib/x86_64-linux-gnu/libgmp.so.10
          libgcrypt.so.20 => /lib/x86_64-linux-gnu/libgcrypt.so.20
            libgpg-error.so.0 => /lib/x86_64-linux-gnu/libgpg-error.so.0
      libspatialite.so.6 => /usr/local/lib/libspatialite.so.6
        librttopo.so.1 => /usr/local/lib/librttopo.so.1
      libfreexl.so.1 => /usr/local/lib/libfreexl.so.1
      libpcre.so.3 => /lib/x86_64-linux-gnu/libpcre.so.3
      libcurl.so.4 => /usr/local/lib/libcurl.so.4
      libltidsdk.so => /home/mj10777/000_links/apps_local/gdal_externals/lib/libltidsdk.so
      libNCSEcw.so.0 => /home/mj10777/000_links/apps_local/gdal_externals/lib/libNCSEcw.so.0
      libNCSEcwC.so.0 => /home/mj10777/000_links/apps_local/gdal_externals/lib/libNCSEcwC.so.0
      libNCSCnet.so.0 => /home/mj10777/000_links/apps_local/gdal_externals/lib/libNCSCnet.so.0
      libNCSUtil.so.0 => /home/mj10777/000_links/apps_local/gdal_externals/lib/libNCSUtil.so.0
      libiconv.so.2 => /usr/local/lib/libiconv.so.2
      libFileGDBAPI.so => /home/mj10777/000_links/apps_local/gdal_externals/lib/libFileGDBAPI.so
        libfgdbunixrtl.so => /usr/local/lib/libfgdbunixrtl.so
      libmysqlclient.so.18 => /usr/lib/x86_64-linux-gnu/libmysqlclient.so.18
    libqgis_analysis.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_analysis.so.2.99.0
      libQt5Concurrent.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Concurrent.so.5
    libqscintilla2_qt5.so.13 => /home/mj10777/000_links/apps_local/libQt571/lib/libqscintilla2_qt5.so.13
    libQt5Network.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Network.so.5
    libQt5Xml.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Xml.so.5
    libQt5PrintSupport.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5PrintSupport.so.5
    libQt5Svg.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Svg.so.5
    libproj.so.0 => /usr/local/lib/libproj.so.0
    libgeos_c.so.1 => /usr/local/lib/libgeos_c.so.1
      libgeos-3.5.0.so => /usr/local/lib/libgeos-3.5.0.so
    libsqlite3.so.0 => /usr/local/lib/libsqlite3.so.0
    libspatialite.so.7 => /usr/local/lib/libspatialite.so.7
    libm.so.6 => /lib/x86_64-linux-gnu/libm.so.6
  libqgis_gui.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_gui.so.2.99.0
    libqca-qt5.so.2 => /home/mj10777/000_links/apps_local/libQt571/lib/libqca-qt5.so.2
  libqgis_core.so.2.99.0 => /home/mj10777/000_links/apps_local/qgis_300/lib/libqgis_core.so.2.99.0
    libspatialindex.so.3 => /usr/local/lib/libspatialindex.so.3
  libQt5Widgets.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Widgets.so.5
  libQt5Gui.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Gui.so.5
  libQt5Core.so.5 => /home/mj10777/000_links/apps_local/libQt571/lib/libQt5Core.so.5
    libglib-2.0.so.0 => /lib/x86_64-linux-gnu/libglib-2.0.so.0
  libstdc++.so.6 => /usr/lib/x86_64-linux-gnu/libstdc++.so.6
  libgcc_s.so.1 => /lib/x86_64-linux-gnu/libgcc_s.so.1
  libc.so.6 => /lib/x86_64-linux-gnu/libc.so.6
```

---
2017-03-22: Mark Johnson, Berlin Germany

---